<?php


use Chekonline\Cashbox\Api;
use Chekonline\Cashbox\Commands\ComplexCommand;
use Chekonline\Cashbox\DocumentType;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;
use Chekonline\Cashbox\Line;
use Chekonline\Cashbox\PayAttribute;

require_once(__DIR__ . '/vendor/autoload.php');

class shopChekonlinePlugin extends shopPlugin
{

    const LOG_PATH = 'shop/plugins/chekonline';
    /**
     * @var Api
     */
    public $client;

    public function allowedCurrency()
    {
        return array('RUB');
    }

    public function init()
    {
        $this->client = new Api($this->getSettings('chekonline_endpoint'));

        $this->client->setCertificate(
            $this->getSettings('chekonline_cert'),
            $this->getSettings('chekonline_key'),
            $this->getSettings('chekonline_cert_password')
        );
    }


    public function writeLog($type, $message)
    {

        if ( ! $this->getSettings('chekonline_log')) {
            return;
        }
        $now  = time();
        $date = date('Y-m-d', $now);
        waLog::log(strval($message), self::LOG_PATH . "/{$date}-{$type}.txt");
    }

    public function debit($params)
    {
        if ($params['action_id'] == 'complete' && ! $this->getSettings('chekonline_status_paid')) {
            return;
        }

        $this->sendReceipt(0, $params);
    }

    public function refund($params)
    {
        $this->sendReceipt(1, $params);
    }

    private function sendReceipt($type, $params)
    {

        try {
            $this->init();
            if ($type === 0) {
                $documentType = DocumentType::DEBIT;
            } else {
                $documentType = DocumentType::REFUND_DEBIT;
            }

            $orderId = $params['order_id'];

            $order = shopPayment::getOrderData($orderId, $this);

            $paymentMethods =$this->getSettings('chekonline_payment_types');
            if (!isset($paymentMethods[$order->params['payment_id']])) {
                return;
            }

            $customer_email = $order->getContactField('email', 'default');
            $customer_phone = $order->getContactField('phone', 'default');

            $contact = ! empty($customer_email) ? $customer_email : $customer_phone;

            $receipt_model = new shopChekonlineReceiptModel();

            $receiptId = $receipt_model->findOrCreate($orderId, $type)['id'];

            $orderItems = $order->items;

            $total = round($order->total * 100);

            $command = new ComplexCommand(strval($receiptId));
            $command->setDevice('auto')
                    ->setNonCash(array(0, $total, 0))
                    ->setPhoneOrEmail($contact)
                    ->setClientId($this->getSettings('chekonline_client_id'))
                    ->setPassword(intval($this->getSettings('chekonline_cashier_password')))
                    ->setGroup($this->getSettings('chekonline_group'))
                    ->setDocumentType($documentType)
                    ->setMaxDocumentsInTurn(intval($this->getSettings('chekonline_docs_in_turn')))
                    ->setTaxMode(pow(2, $this->getSettings('chekonline_tax_mode')));

            $productTaxId = intval($this->getSettings('chekonline_product_vat'));
            foreach ($orderItems as $orderItem) {
                $qty      = round($orderItem['quantity'] * 1000);
                $desc     = $orderItem['name'];
                $subtotal = round($orderItem['total'] * 100);

                if ($subtotal < 0 ) {
                    continue;
                }

                $line = new Line();
                $line->setQty($qty)
                     ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                     ->setTaxId($productTaxId)
                     ->setSubTotal($subtotal)
                     ->setDescription($desc);

                $line->validate();
                $command->addLine($line);
            }

            $deliveryTaxId = intval($this->getSettings('chekonline_delivery_vat'));

            $deliveryTotal = round($order['shipping'] * 100);



            if ($deliveryTotal > 0) {

                $qty      =  1000;
                $desc     = "Доставка: " . $order["shipping_name"];

                $line = new Line(true);
                $line->setQty($qty)
                     ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                     ->setTaxId($deliveryTaxId)
                     ->setSubTotal($deliveryTotal)
                     ->setDescription($desc);

                $line->validate();
                $command->addLine($line);
            }

            $itemTotal = $command->getAmount(true);

            if ($itemTotal !== $total) {
                $command->normalizeBySubTotal($total, true);
            }

            $command->validate();

            $response = $this->client->executeCommand($command);

            if ($response->getHttpStatusCode() != 200 ||
                $response->getApiErrorCode() != 0 ||
                ! empty($response->getHttpError())) {
                $this->writeLog('error', $response);
            }

            $receipt_model->receiptSuccess($receiptId);

            $this->writeLog('response', $response);
            $this->writeLog('request', $response->getRequest());
        } catch (ChekonlineLineException $exception) {
            $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nLine:\n" . var_export($exception->getReceiptLine()->getParam(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            $this->writeLog('error', $logMessage);
        } catch (ChekonlineCommandException $exception) {
            $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nCommand:" . var_export($exception->getCommand()->getParams(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            $this->writeLog('error', $logMessage);
        } catch (ChekonlineSDKException $exception) {
            $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            $this->writeLog('error', $logMessage);
        }

    }
}